@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005, 2006 Marius L. Jøhndal
+  Copyright (C) 2005, 2006, 2007 Marius L. Jøhndal
  
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,7 +15,7 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
-  $Id: castget.c,v 1.15 2006/07/08 09:17:48 mariuslj Exp $
+  $Id: castget.c,v 1.22 2007/02/15 17:53:26 mariuslj Exp $
   
 */
 
@@ -30,6 +30,7 @@
 #include <glib/gstdio.h>
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
 #include <libxml/parser.h>
 #ifdef ENABLE_ID3LIB
 #include <id3.h>
@@ -63,6 +64,9 @@ static int playlist_add(const gchar *playlist_file,
 
 static int verbose = 0;
 static int quiet = 0;
+static int new = 0;
+static int first_only = 0;
+static int resume = 0;
 
 int main(int argc, char **argv)
 {
@@ -79,15 +83,18 @@ int main(int argc, char **argv)
 
     static struct option long_options[] = {
       {"catchup", 0, 0, 'c'},
+      {"first-only", 0, 0, '1'},
       {"help", 0, 0, 'h'},
       {"list", 0, 0, 'l'},
+      {"new-only", 0, 0, 'n'},
+      {"resume", 0, 0, 'r'},
       {"verbose", 0, 0, 'v'},
       {"version", 0, 0, 'V'},
       {"quiet", 0, 0, 'q'},
       {0, 0, 0, 0}
     };
     
-    c = getopt_long(argc, argv, "chlqvV", long_options, &option_index);
+    c = getopt_long(argc, argv, "1chlnqrvV", long_options, &option_index);
 
     if (c == -1)
       break;
@@ -101,6 +108,18 @@ int main(int argc, char **argv)
       op = OP_LIST;
       break;
 
+    case 'n':
+      new = 1;
+      break;
+
+    case 'r':
+      resume = 1;
+      break;
+
+    case '1':
+      first_only = 1;
+      break;
+
     case 'v':
       verbose = 1;
       break;
@@ -126,6 +145,10 @@ int main(int argc, char **argv)
     return 1;
   }
 
+  if (verbose && new) {
+    printf("Fetching new channels only...\n");
+  }
+
   LIBXML_TEST_VERSION;
         
   /* Build the channel directory path and ensure that it exists. */
@@ -185,14 +208,17 @@ int main(int argc, char **argv)
 
 static void usage(void)
 {
-  g_printf("Usage: castget [-c|-l|-V|-h] [-v|-q] [identifier(s)]\n\n");
-  g_printf("  --catchup   -c    Catch up with channels.\n");
-  g_printf("  --list      -l    List available enclosures.\n");
-  g_printf("  --version   -V    Print version number.\n");
-  g_printf("  --help      -h    Print usage information.\n");
+  g_printf("Usage: castget [-c|-l|-V|-h] [-v|-q] [-r] [-1] [-n] [identifier(s)]\n\n");
+  g_printf("  --catchup     -c    Catch up with channels.\n");
+  g_printf("  --list        -l    List available enclosures.\n");
+  g_printf("  --first-only  -1    Only operate on the most recent enclosure in each channel.\n");
+  g_printf("  --new-only    -n    Only operate on new channel(s).\n");
+  g_printf("  --resume      -r    Resume aborted downloads.\n");
+  g_printf("  --version     -V    Print version number.\n");
+  g_printf("  --help        -h    Print usage information.\n");
   g_printf("\n");
-  g_printf("  --verbose   -v    Verbose mode.\n");
-  g_printf("  --quiet     -q    Quiet mode.\n\n");
+  g_printf("  --verbose     -v    Verbose mode.\n");
+  g_printf("  --quiet       -q    Quiet mode.\n\n");
   g_printf("The identifiers identifies the channels affected by the selected operation.\n");
   g_printf("If no identifier is supplied all channels are affect.\n");
 }
@@ -209,64 +235,67 @@ static void update_callback(void *user_data, libcastget_channel_action action,
 {
   struct channel_configuration *c = (struct channel_configuration *)user_data;
 
-  if (!quiet) {
-    switch (action) {
-    case CCA_RSS_DOWNLOAD_START:
+  switch (action) {
+  case CCA_RSS_DOWNLOAD_START:
+    if (!quiet)
       g_printf("Updating channel %s...\n", c->identifier);
-      break;
-
-    case CCA_ENCLOSURE_DOWNLOAD_START:
-      g_assert(channel_info);
-      g_assert(enclosure);
-
-      if (verbose) {
-        if (enclosure->length > 1024*1024*1024) {
-          g_printf(" * Downloading %s (%.1f GB) from %s\n", 
-                   enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0), 
-                   channel_info->title);
-        } else if (enclosure->length > 1024*1024) {
-          g_printf(" * Downloading %s (%.1f MB) from %s\n", 
-                   enclosure->filename, (float)enclosure->length / (1024.0*1024.0),
-                   channel_info->title);
-        } else if (enclosure->length > 1024) {
-          g_printf(" * Downloading %s (%.1f kB) from %s\n", 
-                   enclosure->filename, (float)enclosure->length / 1024.0,
-                   channel_info->title);
-        } else if (enclosure->length > 0) {
-          g_printf(" * Downloading %s (%ld bytes) from %s\n", 
-                   enclosure->filename, enclosure->length,
-                   channel_info->title);
-        } else {
-          g_printf(" * Downloading %s from %s\n",
-                   enclosure->filename,
-                   channel_info->title);
-        }
+    break;
+    
+  case CCA_RSS_DOWNLOAD_END:
+    break;
+    
+  case CCA_ENCLOSURE_DOWNLOAD_START:
+    g_assert(channel_info);
+    g_assert(enclosure);
+    
+    if (verbose) {
+      if (enclosure->length > 1024*1024*1024) {
+        g_printf(" * Downloading %s (%.1f GB) from %s\n", 
+                 enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0), 
+                 channel_info->title);
+      } else if (enclosure->length > 1024*1024) {
+        g_printf(" * Downloading %s (%.1f MB) from %s\n", 
+                 enclosure->filename, (float)enclosure->length / (1024.0*1024.0),
+                 channel_info->title);
+      } else if (enclosure->length > 1024) {
+        g_printf(" * Downloading %s (%.1f kB) from %s\n", 
+                 enclosure->filename, (float)enclosure->length / 1024.0,
+                 channel_info->title);
+      } else if (enclosure->length > 0) {
+        g_printf(" * Downloading %s (%ld bytes) from %s\n", 
+                 enclosure->filename, enclosure->length,
+                 channel_info->title);
+      } else {
+        g_printf(" * Downloading %s from %s\n",
+                 enclosure->filename,
+                 channel_info->title);
       }
-      break;
-
-    case CCA_ENCLOSURE_DOWNLOAD_END:
-      g_assert(channel_info);
-      g_assert(enclosure);
-      g_assert(filename);
-
-      /* Set media tags. */
-      if (enclosure->type && !strcmp(enclosure->type, "audio/mpeg")) {
-#ifdef ENABLE_ID3LIB
-        _id3_check_and_set(filename, c);
-#endif /* ENABLE_ID3LIB */
-      }
-
-      /* Update playlist. */
-      if (c->playlist) {
-        playlist_add(c->playlist, filename);
-
-        if (verbose)
-          printf(" * Added downloaded enclosure %s to playlist %s.\n", 
-                 filename, c->playlist);
-      }
-      break;
     }
-  } 
+    break;
+    
+  case CCA_ENCLOSURE_DOWNLOAD_END:
+    g_assert(channel_info);
+    g_assert(enclosure);
+    g_assert(filename);
+    
+    /* Set media tags. */
+    if (enclosure->type && !strcmp(enclosure->type, "audio/mpeg")) {
+#ifdef ENABLE_ID3LIB
+      if (_id3_check_and_set(filename, c))
+        fprintf(stderr, "Error setting ID3 tag for file %s.\n", filename);
+#endif /* ENABLE_ID3LIB */
+    }
+    
+    /* Update playlist. */
+    if (c->playlist) {
+      playlist_add(c->playlist, filename);
+      
+      if (verbose)
+        printf(" * Added downloaded enclosure %s to playlist %s.\n", 
+               filename, c->playlist);
+    }
+    break;
+  }
 }
 
 static void catchup_callback(void *user_data, libcastget_channel_action action, libcastget_channel_info *channel_info,
@@ -274,21 +303,26 @@ static void catchup_callback(void *user_data, libcastget_channel_action action,
 {
   struct channel_configuration *c = (struct channel_configuration *)user_data;
 
-  if (!quiet) {
-    switch (action) {
-    case CCA_RSS_DOWNLOAD_START:
+  switch (action) {
+  case CCA_RSS_DOWNLOAD_START:
+    if (!quiet)
       g_printf("Catching up with channel %s...\n", c->identifier);
-      break;
+    break;
+    
+  case CCA_RSS_DOWNLOAD_END:
+    break;
 
-    case CCA_ENCLOSURE_DOWNLOAD_START:
-      g_assert(channel_info);
-      g_assert(enclosure);
-
-      if (verbose)
-        g_printf("Catching up on %s (%ld bytes) from %s\n", enclosure->url, enclosure->length,
-                 channel_info->title);
-      break;
-    }
+  case CCA_ENCLOSURE_DOWNLOAD_START:
+    g_assert(channel_info);
+    g_assert(enclosure);
+    
+    if (verbose)
+      g_printf("Catching up on %s (%ld bytes) from %s\n", enclosure->url, enclosure->length,
+               channel_info->title);
+    break;
+    
+  case CCA_ENCLOSURE_DOWNLOAD_END:
+    break;
   }
 }
 
@@ -297,20 +331,41 @@ static void list_callback(void *user_data, libcastget_channel_action action, lib
 {
   struct channel_configuration *c = (struct channel_configuration *)user_data;
 
-  if (!quiet) {
-    switch (action) {
-    case CCA_RSS_DOWNLOAD_START:
-      g_printf("Listing channel %s...\n", c->identifier);
-      break;
-
-    case CCA_ENCLOSURE_DOWNLOAD_START:
-      g_assert(channel_info);
-      g_assert(enclosure);
-
-      g_printf("* %s (%ld bytes) from %s\n", enclosure->url, enclosure->length, 
+  switch (action) {
+  case CCA_RSS_DOWNLOAD_START:
+    g_printf("Listing channel %s...\n", c->identifier);
+    break;
+    
+  case CCA_RSS_DOWNLOAD_END:
+    break;
+    
+  case CCA_ENCLOSURE_DOWNLOAD_START:
+    g_assert(channel_info);
+    g_assert(enclosure);
+    
+    if (enclosure->length > 1024*1024*1024) {
+      g_printf(" * %s (%.1f GB) from %s\n", 
+               enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0), 
                channel_info->title);
-      break;
+    } else if (enclosure->length > 1024*1024) {
+      g_printf(" * %s (%.1f MB) from %s\n", 
+               enclosure->filename, (float)enclosure->length / (1024.0*1024.0),
+               channel_info->title);
+    } else if (enclosure->length > 1024) {
+      g_printf(" * %s (%.1f kB) from %s\n", 
+               enclosure->filename, (float)enclosure->length / 1024.0,
+               channel_info->title);
+    } else if (enclosure->length > 0) {
+      g_printf(" * %s (%ld bytes) from %s\n", 
+               enclosure->filename, enclosure->length, channel_info->title);
+    } else {
+      g_printf(" * %s from %s\n", enclosure->filename, channel_info->title);
     }
+    
+    break;
+    
+  case CCA_ENCLOSURE_DOWNLOAD_END:
+    break;
   }
 }
 
@@ -387,9 +442,17 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
   channel_filename = g_strjoin(".", identifier, "xml", NULL);
   channel_file = g_build_filename(channel_directory, channel_filename, NULL);
   g_free(channel_filename);
-        
+
+  if (new && access(channel_file, F_OK) == 0) {
+    /* If we are only fetching new channels, skip the channel if there is
+       already a channel file present. */
+
+    channel_configuration_free(channel_configuration);
+    return 0;
+  }
+
   c = libcastget_channel_new(channel_configuration->url, channel_file, 
-                             channel_configuration->spool_directory);
+                             channel_configuration->spool_directory, resume);
   g_free(channel_file);
 
   if (!c) {
@@ -401,15 +464,18 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
     
   switch (op) {
   case OP_UPDATE:
-    libcastget_channel_update(c, channel_configuration, update_callback);
+    libcastget_channel_update(c, channel_configuration, update_callback, 
+                              0, 0, first_only, resume);
     break;
             
   case OP_CATCHUP:
-    libcastget_channel_catchup(c, channel_configuration, catchup_callback);
+    libcastget_channel_update(c, channel_configuration, catchup_callback,
+                              1, 0, first_only, 0);
     break;
             
   case OP_LIST:
-    libcastget_channel_list(c, channel_configuration, list_callback);
+    libcastget_channel_update(c, channel_configuration, list_callback,
+                              1, 1, first_only, 0);
     break;
   }
           
